#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""Unittest for testing the package entry points"""

import subprocess
import unittest


class TestEntryPoints(unittest.TestCase):

    def setUp(self) -> None:
        """Run before every test method"""
        pass

    def tearDown(self) -> None:
        """Run after every test method"""
        pass

    def test_help(self) -> None:
        """Test help command of package"""
        cmd = ['generate-versioned-pages', '--help']
        result = subprocess.run(cmd, stdout=subprocess.PIPE)
        self.assertIn(
            'Generate index page of available versioned pages',
            result.stdout.decode('utf-8')
        )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
Generate index page with links to all previously archived folders during a tag
build
"""

import argparse
import json
import logging
from dataclasses import dataclass, field
from re import sub
from gitlab import Gitlab
from gitlab.v4.objects.commits import ProjectCommit
from gitlab.v4.objects.projects import Project
from gitlab.v4.objects.tags import ProjectTag
from jinja2 import Environment, FileSystemLoader
from jinja2.environment import Template
from sys import stdout
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional

from .version import __version__


def parse_arguments() -> argparse.Namespace:
    """
    Parse CLI arguments.
    :raise      argparse.ArgumentError  Argparse error
    :return:    argparse object
    """
    parser = argparse.ArgumentParser(description="""
    Generate index page of available versioned pages
    """, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    # default arguments
    parser.add_argument('-d', '--debug',
                        action='store_true',
                        help='Output logger messages to stderr')
    parser.add_argument('-v',
                        default=0,
                        action='count',
                        dest='verbosity',
                        help='Set level of verbosity, default is CRITICAL')
    parser.add_argument('--version',
                        action='version',
                        version='%(prog)s {version}'.
                                format(version=__version__),
                        help="Print version of package and exit")

    # specific arguments
    parser.add_argument('--url',
                        default='https://gitlab.com',
                        help='Basic GitLab URL')
    parser.add_argument('--private-token',
                        default=None,
                        help='Project Access Token with API scope')
    parser.add_argument('--project-id',
                        default=None,
                        required=True,
                        help='Project ID')
    parser.add_argument('--job-name',
                        default=None,
                        required=True,
                        help='Job name which generated the public folder')
    parser.add_argument('--output-dir',
                        default=Path('public').expanduser().resolve(),
                        type=Path,
                        help='Output directory of versioned pages index file')
    parser.add_argument('--pages-base-url',
                        default=None,
                        help='URL of GitLab page, see Settings -> Pages')
    parser.add_argument('--create-version-info-file',
                        action='store_true',
                        help='Create version info JSON file in output folder')
    parser.add_argument('--template-file',
                        type=lambda x: parser_valid_file(parser=parser, arg=x),
                        help='Path to custom index template file')

    parsed_args = parser.parse_args()

    return parsed_args


def parser_valid_file(parser: argparse.ArgumentParser, arg: str) -> Path:
    """
    Determine whether file exists.

    :param      parser:                 The parser
    :type       parser:                 parser object
    :param      arg:                    The file to check
    :type       arg:                    str
    :raise      argparse.ArgumentError: Argument is not a file

    :returns:   Input file path, parser error is thrown otherwise.
    :rtype:     Path
    """
    if not Path(arg).is_file():
        parser.error("The file {} does not exist!".format(arg))
    else:
        return Path(arg).resolve()


@dataclass
class TagInfo:
    tag: ProjectTag
    commit: ProjectCommit
    created_at: datetime
    job_id: int = -1
    pages_url: str = ''
    job_ids: List[Dict[str, int]] = field(default_factory=list)


def get_project(url: str, private_token: str, project_id: int) -> Project:
    """
    Get the GitLab project.

    :param      url:            The url
    :type       url:            str
    :param      private_token:  The private token
    :type       private_token:  str
    :param      project_id:     The project identifier
    :type       project_id:     int

    :returns:   The project.
    :rtype:     Project
    """
    gl = Gitlab(url=url, private_token=private_token)
    project = gl.projects.get(project_id)

    return project


def get_project_tags(project: Project,
                     job_name: str,
                     web_url: str) -> List[TagInfo]:
    """
    Get all project tags.

    :param      project:   The project
    :type       project:   Project
    :param      job_name:  The job name
    :type       job_name:  str
    :param      web_url:   The web url
    :type       web_url:   str

    :returns:   The project tags.
    :rtype:     List[TagInfo]
    """
    tags: List[TagInfo] = []

    for tag in project.tags.list(all=True, as_list=False):
        tag_info = TagInfo(
            tag=tag,    # type: ignore
            commit=project.commits.get(tag.attributes['commit']['id']),
            created_at=datetime.strptime(
                sub(pattern=r'\+.*', repl='', string=tag.commit['created_at']),
                "%Y-%m-%dT%H:%M:%S.%f"
            ),
        )

        get_pipeline_job(
            project=project,
            tag_info=tag_info,
            job_name=job_name,
            web_url=web_url
        )

        tags.append(tag_info)

    return tags


def get_pipeline_job(project: Project,
                     tag_info: TagInfo,
                     job_name: str,
                     web_url: str) -> None:
    """
    Get the pipeline job informations and set the tag info values.

    :param      project:   The project
    :type       project:   Project
    :param      tag_info:  The tag information
    :type       tag_info:  TagInfo
    :param      job_name:  The job name
    :type       job_name:  str
    :param      web_url:   The web url
    :type       web_url:   str
    """
    pages_url: str = ''
    job_id: int = -1
    pipeline_ids: List[Dict[str, int]] = []

    last_pipeline_id = tag_info.commit.last_pipeline['id']
    pipeline = project.pipelines.get(last_pipeline_id)

    for job in pipeline.jobs.list(all=True, as_list=False):
        pipeline_ids.append({job.name: job.id})

        if job.status == "success" and (
            job_name is None or job.name == job_name
        ):
            job_id = job.id
            pages_url = get_artifact_url(
                web_url=web_url,
                job_id=job_id,
                folder='public',
                index_file='index.html'
            )

    tag_info.job_id = job_id
    tag_info.job_ids = pipeline_ids
    tag_info.pages_url = pages_url


def get_artifact_url(web_url: str,
                     job_id: int,
                     folder: str,
                     index_file: str) -> str:
    """
    Create the artifact URL.

    :param      web_url:     The web url
    :type       web_url:     str
    :param      job_id:      The job identifier
    :type       job_id:      int
    :param      folder:      The folder
    :type       folder:      str
    :param      index_file:  The index file
    :type       index_file:  str

    :returns:   The artifact URL.
    :rtype:     str
    """
    url = (
        '{web_url}/-/jobs/{id}/artifacts/{external}{folder}/{index_file}'.
        format(
            web_url=web_url,
            id=job_id,
            external='external_file/' if '.gitlab.io' not in web_url else '',
            folder=folder,
            index_file=index_file
        )
    )

    return url


def save_version_info_file(tag_list: List[TagInfo], file_path: Path) -> None:
    """
    Save a version information file.

    :param      tag_list:   The tag list
    :type       tag_list:   List[TagInfo]
    :param      file_path:  The file path
    :type       file_path:  Path
    """
    version_info = []

    for tag in tag_list:
        info = dict(tag.tag.attributes)
        info['pages_url'] = tag.pages_url
        info['job_id'] = tag.job_id
        info['commit_info'] = tag.commit.attributes
        version_info.append(info)

    save_file(
        content=json.dumps(version_info, indent=4, sort_keys=True),
        path=file_path
    )


def get_template_file(file_name: str,
                      template_folder: Optional[Path] = None) -> Template:
    """
    Get the Jinja2 template file.

    :param      file_name:        The template file name
    :type       file_name:        str
    :param      template_folder:  The template folder
    :type       template_folder:  Path

    :returns:   The template.
    :rtype:     Template
    """
    if template_folder is None:
        template_folder = Path(__file__).parent / "templates"

    environment = Environment(loader=FileSystemLoader(template_folder))
    template = environment.get_template(file_name)

    return template


def save_file(content: str, path: Path) -> None:
    """
    Save data to a file.

    :param      content:  The content
    :type       content:  str
    :param      path:     The path
    :type       path:     Path
    """
    if not path.parent.exists():
        create_output_directory(path=path.parent)

    with open(path, 'w', encoding='utf-8') as f:
        f.write(content)


def create_output_directory(path: Path) -> None:
    """
    Create the output directory.

    :param      path:  The path to the output directory
    :type       path:  Path
    """
    Path(path).mkdir(parents=True, exist_ok=True)


def create_html_files(tag_list: List[TagInfo],
                      path: Path,
                      template: Optional[Path] = None) -> None:
    """
    Create all HTML files.

    :param      tag_list:  The tag list
    :type       tag_list:  List[TagInfo]
    :param      path:      The path to the output folder
    :type       path:      Path
    :param      template:  Path to custom template file
    :type       template:  Optional[Path]
    """
    file_name = 'index.html'
    template_folder = None

    if template is not None:
        file_name = template.name
        template_folder = template.parent

    index_template = get_template_file(file_name=file_name,
                                       template_folder=template_folder)

    tag_base_url = sub(
        pattern=r'\/-\/commit\/.*',
        repl='/-/tags/',
        string=tag_list[0].commit.web_url
    )
    index_content = index_template.render(
        items=tag_list,
        tag_base_url=tag_base_url
    )
    save_file(content=index_content, path=path / file_name)


def main() -> None:
    # parse CLI arguments
    args = parse_arguments()

    log_levels = {
        0: logging.CRITICAL,
        1: logging.ERROR,
        2: logging.WARNING,
        3: logging.INFO,
        4: logging.DEBUG,
    }
    custom_format = '[%(asctime)s] [%(levelname)-8s] [%(filename)-15s @'\
                    ' %(funcName)-15s:%(lineno)4s] %(message)s'
    logging.basicConfig(level=logging.INFO,
                        format=custom_format,
                        stream=stdout)
    logger = logging.getLogger(__name__)
    logger.setLevel(level=log_levels[min(args.verbosity,
                                     max(log_levels.keys()))])
    logger.disabled = not args.debug

    logger.debug(args)
    url = args.url
    private_token = args.private_token
    project_id = args.project_id
    job_name = args.job_name
    output_path = args.output_dir
    pages_base_url = args.pages_base_url
    create_version_info_file = args.create_version_info_file
    template_file = args.template_file

    project = get_project(
        url=url,
        private_token=private_token,
        project_id=project_id
    )

    create_output_directory(path=output_path)

    if pages_base_url is None:
        pages_base_url = 'https://{owner}.gitlab.io/-/{name}'.format(
            owner=project.attributes['namespace']['name'],
            name=project.attributes['name']
        )

    if pages_base_url is not None:
        web_url = pages_base_url
    else:
        web_url = project.attributes['web_url']

    # get all tags of the project
    tag_list = get_project_tags(
        project=project,
        job_name=job_name,
        web_url=web_url
    )

    if create_version_info_file:
        save_version_info_file(
            tag_list=tag_list,
            file_path=output_path / 'versions.json'
        )

    create_html_files(
        tag_list=tag_list,
        path=output_path,
        template=template_file
    )


if __name__ == '__main__':
    main()  # pragma: no cover



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""Unittest for testing the documentation index page"""

import argparse
from datetime import datetime
import gitlab
import jinja2
import logging
from nose2.tools import params
from pathlib import Path
from sys import stdout
from typing import Optional
import unittest
from unittest.mock import patch, MagicMock

from lightweight_versioned_gitlab_pages import generate


class TestGenerate(unittest.TestCase):

    def setUp(self) -> None:
        """Run before every test method"""
        # define a format
        custom_format = '[%(asctime)s] [%(levelname)-8s] [%(filename)-15s @'\
                        ' %(funcName)-15s:%(lineno)4s] %(message)s'

        # set basic config and level for all loggers
        logging.basicConfig(level=logging.INFO,
                            format=custom_format,
                            stream=stdout)

        # create a logger for this TestSuite
        self.test_logger = logging.getLogger(__name__)
        self.package_logger = logging.getLogger('GeneratePage')

        # set the test logger level
        self.test_logger.setLevel(logging.DEBUG)
        self.package_logger.setLevel(logging.DEBUG)

        self._tests_directory = (Path(__file__).parent).resolve()

    def tearDown(self) -> None:
        """Run after every test method"""
        pass

    @patch('sys.argv', ['main', '--project-id', '1234', '--job-name', 'carl'])
    def test_parse_arguments_default(self):
        expectation = {
            'debug': False,
            'verbosity': 0,
            'url': 'https://gitlab.com',
            'private_token': None,
            'project_id': '1234',
            'job_name': 'carl',
            'output_dir':
                Path(__file__).parent.parent.expanduser().resolve() / 'public',
            'pages_base_url': None,
            'create_version_info_file': False,
            'template_file': None,
        }
        args = generate.parse_arguments()
        self.test_logger.debug(args)
        args_as_dict = vars(args)

        for k, v in expectation.items():
            self.assertEqual(args_as_dict[k], v)

    @patch(
        'sys.argv', [
            'main', '--project-id', '1234', '--job-name', 'carl',
            '--url', 'http://git.my-url.com',
            '--private-token', 'qwertz1234',
            '--output-dir', 'one/dir',
            '--template-file', 'tests/data/index.txt',
            '--debug', '-vvvv'
        ]
    )
    def test_parse_arguments_specific(self):
        expectation = {
            'debug': True,
            'verbosity': 4,
            'url': 'http://git.my-url.com',
            'private_token': 'qwertz1234',
            'project_id': '1234',
            'job_name': 'carl',
            'output_dir': Path('one/dir'),
            'pages_base_url': None,
            'create_version_info_file': False,
            'template_file': Path(__file__).parent / 'data' / 'index.txt',
        }
        args = generate.parse_arguments()
        self.test_logger.debug(args)
        args_as_dict = vars(args)

        for k, v in expectation.items():
            self.assertEqual(args_as_dict[k], v)

    def test_parser_valid_file(self):
        parser = argparse.ArgumentParser()
        file_path = __file__

        result = generate.parser_valid_file(parser=parser, arg=file_path)
        self.assertEqual(result, Path(file_path).resolve())

    def test_parser_invalid_file(self):
        parser = argparse.ArgumentParser()
        file_path = 'asdf.qwertz'

        with self.assertRaises(SystemExit) as context:
            generate.parser_valid_file(parser=parser, arg=file_path)

        self.assertEqual('2', str(context.exception))

    def test_get_project(self):
        url = 'https://gitlab.com'
        token = None
        project_id = 43170198

        project = generate.get_project(
            url=url,
            private_token=token,
            project_id=project_id
        )

        self.assertIsInstance(project, gitlab.v4.objects.projects.Project)
        self.assertEqual(project.id, project_id)

    def test_get_project_tags(self):
        project = MagicMock()

        first_project_tag = MagicMock()
        second_project_tag = MagicMock()

        first_project_tag.attributes = {'commit': {'id': 'bcf01494'}}
        first_project_tag.commits.get = 'bcf01494'
        first_project_tag.commit = {
            'created_at': '2023-02-03T15:04:40.000+00:00'
        }

        second_project_tag.attributes = {'commit': {'id': '23fb4d72'}}
        second_project_tag.commits.get = '23fb4d72'
        second_project_tag.commit = {
            'created_at': '2023-02-03T14:15:26.000+00:00'
        }

        project_tags = [first_project_tag, second_project_tag]
        project.tags.list = MagicMock(return_value=project_tags)

        with patch.object(generate, 'get_pipeline_job', return_value=None):
            tags = generate.get_project_tags(
                project=project,
                job_name='carl',
                web_url='asdf'
            )
        self.test_logger.debug(tags)
        self.assertIsInstance(tags, list)
        self.assertEqual(len(tags), 2)
        for tag in tags:
            self.assertIsInstance(tag, generate.TagInfo)
            self.assertIsInstance(tag.created_at, datetime)
            self.test_logger.debug(tag)

    @unittest.skip("Not yet implemented")
    def test_get_pipeline_job(self):
        pass

    def test_get_artifact_url(self):
        web_url = 'https://brainelectronics.gitlab.io/-/asdf'
        job_id = 1234
        folder = 'public'
        index_file = 'index.html'
        expectation = \
            'https://brainelectronics.gitlab.io/-/asdf/-/jobs/1234/artifacts/public/index.html'   # noqa: E501

        result = generate.get_artifact_url(
            web_url=web_url,
            job_id=job_id,
            folder=folder,
            index_file=index_file
        )
        self.assertEqual(result, expectation)

        web_url = 'https://gitlab.com/brainelectronics/quertz'
        job_id = 42
        folder = 'other'
        index_file = 'root.php'
        expectation = \
            'https://gitlab.com/brainelectronics/quertz/-/jobs/42/artifacts/external_file/other/root.php'   # noqa: E501

        result = generate.get_artifact_url(
            web_url=web_url,
            job_id=job_id,
            folder=folder,
            index_file=index_file
        )
        self.assertEqual(result, expectation)

    @unittest.skip("Not yet implemented")
    def test_save_version_info_file(self):
        pass

    @params(
        ('index.html', None),           # use template folder of package
        ('not_existing.html', None),    # use template folder of package
        ('index.txt', 'data/'),         # use custom folder path
        ('not_existing.txt', 'data/'),  # use custom folder path
    )
    def test_get_template_file(self,
                               file_name: str,
                               template_folder: Optional[str]):
        if template_folder is not None:
            template_folder = self._tests_directory / template_folder

        if file_name.startswith('not_existing'):
            with self.assertRaises(jinja2.exceptions.TemplateNotFound) as ctx:
                template = generate.get_template_file(
                    file_name=file_name,
                    template_folder=template_folder
                )

            self.assertEqual(file_name, str(ctx.exception))
        else:
            template = generate.get_template_file(
                file_name=file_name,
                template_folder=template_folder
            )

            self.assertIsInstance(template, jinja2.environment.Template)

    @unittest.skip("Not yet implemented")
    def test_save_file(self):
        pass

    @unittest.skip("Not yet implemented")
    def test_create_output_directory(self):
        pass

    @unittest.skip("Not yet implemented")
    def test_create_html_files(self):
        pass

    '''
    @patch('sys.argv', ['main', '--debug'])
    def test_parse_arguments_debug(self) -> None:
        """Test parsing command line arg debug option"""
        args = generate.parse_arguments()

        self.assertTrue(args.debug)

    @params(
        (None, 0),
        ('-v', 1),
        ('-vv', 2),
        ('-vvv', 3),
        ('-vvvv', 4),
        ('-vvvvv', 5),
    )
    def test_parse_arguments_verbosity(self,
                                       verbose_option: str,
                                       expectation: int) -> None:
        """
        Test parsing command line arg debug option

        :param      verbose_option:  The verbose option
        :type       verbose_option:  str
        :param      expectation:     The verbosity expectation value
        :type       expectation:     int
        """
        cmd_args = ['main', verbose_option] if verbose_option else ['main']
        with patch('sys.argv', cmd_args):
            args = generate.parse_arguments()

        self.assertEqual(args.verbosity, expectation)

    @patch('argparse.ArgumentParser.parse_args',
           return_value=argparse.Namespace(project_id=1234, job_name='carl'))
    def test_main(self, mock_pargse_args) -> None:
        generate.main()
    '''


if __name__ == '__main__':
    unittest.main()
